/* Tic-Tac-Toe */

#include <stdio.h>
#include <stdlib.h>

struct Row {
	int square[3];		/* the 3 squares that make up the row */
	char *name;			/* a description of the row */
};

int Board[9];			/* the actual board */
						/* 0 => empty, 1 => X, 2 => O */
char *Marker = " XO";	/* what markers go on the board */           
int who=1; 				/* who is 1 for X, and 2 for O */

struct Row rows[8] = {	/* all possible 3 in a rows */
	/* first do rows */
	0, 1, 2 , "top row",
	3, 4, 5, "middle row",
	6, 7, 8, "bottom row",
	/* now do columns */
	0, 3, 6, "left column",
	1, 4, 7, "middle column",
	2, 5, 8, "right column",
	/* finally, do the diagonals */
	0, 4, 8, "upper left diagonal", /* I just made these names up */
	2, 4, 6, "upper right diagonal"
};

/* function prototypes */
void print_board_positions(void);
void print_board_status(void);
void print_board(char *pos);
int make_move(void);
int analyze_move(void);
int analyze_row(int row);

int
main()
{
	print_board_positions();
	while (1)
	{ /* loop forever */
		print_board_status();
		if (make_move()) break;		/* we are done */
		if (analyze_move()) break;	/* check for 3 in a row */
	}
	
	printf("Thanks for playing!\n");
	return (0);
}

void
print_board_positions(void)
{ /* print out the board positions */
	static char *positions = "012345678";	/* what the positions are */

	printf("Board positions are ...\n");	
	/* print board with position numbers instead of X's and O's */
	print_board(positions);	
}

void
print_board_status(void)	
{
	char str[10];	/* string for printing board status */
	int i, n;

	/* first, build a string representing current status */	
	for (i=0; i < 9; i++)
	{ /* loop through board */
		n = Board[i];			/* who is in this position */
		str[i] = Marker[n];		/* set to appropriate marker */
	}
	
	printf("Current board is:\n");
	
	/* now print this board */
	print_board(str);
}

void
print_board(char *pos)
{ /* display the board we are being passed */
	printf(" %c | %c | %c \n", pos[0], pos[1], pos[2]);
	printf("---+---+---+---\n");
	printf(" %c | %c | %c \n", pos[3], pos[4], pos[5]);
	printf("---+---+---+---\n");
	printf(" %c | %c | %c \n", pos[6], pos[7], pos[8]);
} 

int
make_move(void)
{
	char buf[BUFSIZ];
	int move;

	printf("Enter move for %c, or 'q' to quit, 'h' for help: ", Marker[who]);
	gets(buf);
	if (buf[0] == 'q') return 1;	/* done */
	if (buf[0] == 'h')
	{ /* help the user out */
		print_board_positions();
		return 0;
	}
	move = atoi(buf);
	
	/* validate user's move */
	if (move < 0 || move > 8)
	{
		printf("Move must be between 0 and 8\n");
		return 0;	/* try again */
	}
	if (Board[move] != 0)
	{ /* square not empty */
		printf("Position %d isn't available\n", move);
		return 0;
	}
	
	/* move valid = make it */
	Board[move] = who;	/* indicate this move */
	
	/* change who moves next */
	if (who == 1) who = 2;
	else who = 1;

	return 0;	/* normal return */	
}

int
analyze_move(void)
{ /* check for 3 in a row */
	int i, ret=0;	/* loop variable and return code */
	
	for (i=0; i < 8; i++)
	{ /* loop through all possible rows */
		printf("Checking %s ... ", rows[i].name);
		/* 
		 * analyze_row will return 0, 1, or 2
		 * 0 if nobody won
		 * 1 if X won
		 * 2 if O won
		 */
		ret = analyze_row(i);	/* see if somebody won */
		if (ret != 0)
		{ /* somebody won */
			printf("%c won!\n", Marker[ret]);
		 	return ret;
		 }
		 else printf("nobody won\n");
	}

	return 0; /* nobody won */
}

int
analyze_row(int row)
{
	int numx = 0, numo = 0, i, pos, n;

	/*
	 * count the number of X's and O's in this row
	 * if we get 3 X's or 3 O's, then that person won
	 */	
	for (i=0; i < 3; i++)
	{ /* loop through all 3 cells */
		pos = rows[row].square[i];	/* what board position */
		n = Board[pos];				/* who has moved there */
		switch(n)
		{
			case 1: numx++; break;	/* X was there */
			case 2: numo++; break;	/* O was there */
		} /* we don't care about zero */
	}
	
	/* see if somebody won */
	if (numx == 3) return 1;		/* X won - 3 in a row */
	if (numo == 3) return 2;		/* O won - 3 in a row */
	return 0;						/* nobody won */
}

		